<?php
//============================================================+
// File name   : example_015.php
// Begin       : 2008-03-04
// Last Update : 2009-03-18
// 
// Description : Example 015 for TCPDF class
//               Bookmarks (Table of Content)
// 
// Author: Nicola Asuni
// 
// (c) Copyright:
//               Nicola Asuni
//               Tecnick.com s.r.l.
//               Via Della Pace, 11
//               09044 Quartucciu (CA)
//               ITALY
//               www.tecnick.com
//               info@tecnick.com
//============================================================+

/**
 * Creates an example PDF TEST document using TCPDF
 * @package com.tecnick.tcpdf
 * @abstract TCPDF - Example: Bookmarks (Table of Content)
 * @author Nicola Asuni
 * @copyright 2004-2009 Nicola Asuni - Tecnick.com S.r.l (www.tecnick.com) Via Della Pace, 11 - 09044 - Quartucciu (CA) - ITALY - www.tecnick.com - info@tecnick.com
 * @link http://tcpdf.org
 * @license http://www.gnu.org/copyleft/lesser.html LGPL
 * @since 2008-03-04
 */

require_once("../common/global_set.php");
require_once("../common/global_func.php");
require_once("../common/global_class.php");
require_once("../common/User.php");
require_once("../include/loadUserFromSession.php");

require_once('../tcpdf/config/lang/eng.php');
require_once('../tcpdf/tcpdf.php');

$book_id = requestToSession("book_id");
$book = getLineBySQL("select a.*,b.user_name from book a left join users b on a.user_id=b.user_id where a.book_id=$book_id");

// create new PDF document
$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false); 

// set document information
$pdf->SetCreator(PDF_CREATOR);
$pdf->SetAuthor($book->user_name);
$pdf->SetTitle($book->book_name);
$pdf->SetSubject($book->book_name);
$pdf->SetKeywords($book->book_name);

// set default header data
$pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE, PDF_HEADER_STRING);


// set header and footer fonts
$pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

// set default monospaced font
// $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

//set margins
$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

//set auto page breaks
$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

//set image scale factor
$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO); 

//set some language-dependent strings
$pdf->setLanguageArray($l); 

// ---------------------------------------------------------

// set font
$sqlstr = "select book_name from book where book_id=$book_id";
$book_name = getFirstVal($sqlstr);

$book_name = mb_convert_encoding($book_name,"utf-8","GBK") ;
// define ('PDF_HEADER_TITLE', $book_name);

$pdf->AddPage();
$pdf->Bookmark($book_name, 1, 0);

$sqlstr = "select a.*,b.content from book_page a left join page_content b on a.page_id=b.page_id where a.book_id=$book_id order by a.sequence";
$matrix = getResult($sqlstr);
for($i=0;$i<count($matrix);$i++){
	$page_id = $matrix[$i]["page_id"];
	$parent_id = $matrix[$i]["parent_id"];
	$chapter_no = $matrix[$i]["chapter_no"];
	$subject = $matrix[$i]["subject"];

	$subject = mb_convert_encoding($subject,"utf-8","GBK") ;

	$htmlcontent = $matrix[$i]["content"];
	$htmlcontent = mb_convert_encoding($htmlcontent,"utf-8","GBK") ;

	$pdf->AddPage();
	$pdf->Bookmark($subject, 1, 1);
//	$pdf->Cell(0, 10, mb_convert_encoding($subject,"utf-8","GBK"), 0, 1, 'L');

	$pdf->SetFont('stsongstdlight', 'B', 20);
	$title = "<h1>".$chapter_no." ".$subject."</h1>";
	$pdf->writeHTML($subject, true, 0, true, 0);
	$pdf->writeHTML("<br>", true, 0, true, 0);

	$pdf->SetFont('stsongstdlight', '', 12);
	$pdf->writeHTML($htmlcontent, true, 0, true, 0);
}


// ---------------------------------------------------------

//Close and output PDF document
$pdf->Output('export_'.$book_id.'.pdf', 'I');

//============================================================+
// END OF FILE                                                 
//============================================================+
?>