<?php

class HTMLTag{
	var $tagName ;
	var $attributes = array();
    var $degree = 0;
    var $children = array();
    
    static $_CRLF = "\r\n";
    static $_TAB = "\t";
	static $_COMMA = "_CMA_";
	static $_EQUALS = "_EQ_";

	function HTMLTag($tagName,$attrs){
		$this->tagName = $tagName ;
		$this->initAttributes($attrs) ;
	}

	public function attrSafeReplace($attrs){
		$attrs = str_replace(",,",HTMLTag::$_COMMA,$attrs);
		$attrs = str_replace("==",HTMLTag::$_EQUALS,$attrs);
		return $attrs;
	}

	public function attrSafeReplaceBack($value){
		$value = str_replace(HTMLTag::$_COMMA,",",$value);
		$value = str_replace(HTMLTag::$_EQUALS,"=",$value);
		return $value;
	}

	public function initAttributes($attrs){
		if($attrs == ""){
			return ;
		}

		$attrs = $this->attrSafeReplace($attrs);

		$matrix = explode(",",$attrs);
		if(count($matrix) == 0){
			return ;
		}
		for($i=0;$i<count($matrix);$i++){
			if($matrix[$i] == ""){
				//echo "----".$attrs."-------";
				continue ;
			}
			$array = explode("=",$matrix[$i]);
			if(count($array) >= 2){
				$value = $this->attrSafeReplaceBack($array[1]);

				$this->setAttribute($array[0],$value);
			}else{
				//echo count($array) ;
				//echo "----".$matrix[$i]."-------";
			}
		}
	}
	
	public function getHTMLBegin(){
		$html  = HTMLTag::$_CRLF ;
		
		$html .= $this->getTabs();
		$html .= "<".$this->tagName;

		while(list($name,$value)=each($this->attributes)){
			if($value == ""){
				continue ;
			}

			$html .= " ".$name."=\"";
			$html .= $this->replaceValue($value) ;
			$html .= "\"";
		}

		if(count($this->children) == 0){
			$html .= "/";
		}
		$html .= ">";	
		
		return $html ;
	}
	
	public function getHTMLEnd(){
		if(count($this->children) == 0){
			return "";
		}

		$html  = HTMLTag::$_CRLF;
		$html .= $this->getTabs();
		return $html . "</" . $this->tagName . ">";
	}
	
	public function getTabs(){
		if($this->degree <= 0){
			return "";
		}
		$html = "";
		for($i=0;$i<$this->degree;$i++){
			$html .= HTMLTag::$_TAB ;
		}
		return $html ;
	}
	
	public function add($node){
		array_push($this->children, $node) ;
	}

	public function append($node){
		array_push($this->children, $node) ;
		return $this ;
	}

	public function appendToHeader($node){
		$temp = array();
		array_push($temp, $node) ;
		$this->children = array_merge ($temp, $this->children);
		return $this ;
	}

	public function appendHTML($html){
		if($html == ""){
			return ;
		}
		// echo $html ;
		$node = new HTML($html);
		return $this->append($node);
	}

	public function appendText($text){
		$node = new TextNode($text);
		return $this->append($node);
	}

	public function last(){
		$count = count($this->children) ;
		if($count <= 0){
			return $this;
		}else{
			return $this->children[$count-1]->last();
		}
	}
    
    public function getHTML(){
		$html  = $this->getHTMLBegin();
		
		if(count($this->children) > 0){
			for($i=0;$i<count($this->children);$i++){
				$node = $this->children[$i];
				$node->setDegree($this->degree + 1);
				$html .= $node->getHTML();
			}
		}
		$html .= $this->getHTMLEnd();
		
		return $html ;
    }

	public function show(){
		echo $this->getHTML();
	}
    
    public function getAttributes(){
    	return $this->attributes ;
    }
    
    public function setAttributes($attributes){
    	$this->attributes = $attributes ;
    }

	public function setAttribute($name,$value){
		$this->attributes[$name] = $value ;
	}

	public function getAttribute($name){
		if(array_key_exists($name,$this->attributes)){
			return $this->attributes[$name];
		}else{
			return "";
		}
	}

	public function hasAttribute($name){
		return array_key_exists($name,$this->attributes);
	}

	public function getDegree(){
    	return $this->degree ;
    }
    
    public function setDegree($degree){
    	$this->degree = $degree ;
		// echo $this->tagName . " degree is ". $this->degree . "<br>";
    }

	public function replaceValue($value){
    	return str_replace("\"","&quot;",$value) ;
    }
}

class Blank extends HTMLTag{
	function Blank(){
		$this->tagName = "";
	}
    
	public function getHTMLBegin(){		
		return "";
	}

	public function getHTMLEnd(){		
		return "";
	}
}
?>